using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;


namespace ImageProcess
{
    /// <summary>
    /// Holds the bitmap arrangement of raw pixel data
    /// </summary>
    public struct PixelData
    {
        public byte blue;
        public byte green;
        public byte red;
    }


    public unsafe class ProcessBitmaps
    {

        private static Bitmap sourceBitmap = null;
        private static Bitmap maskBitmap = null;
        private static Rectangle maskRect;

        private static BitmapData sourceBitmapData = null;
        private static BitmapData maskBitmapData = null;

        private static Byte* sourcepBase = null;
        private static Byte* maskpBase = null;

        private static int sourceWidth;
        private static int maskWidth;

        /// <summary>
        /// Process the bitmap, converting all black pixels (0,0,0) to transparent 
        /// (the color of pixel (0,0)
        /// </summary>
        /// <param name="source">source image (unchanged)</param>
        /// <returns>processed image</returns>
        public static Bitmap MakeBlackTransparent(Bitmap source)
        {
            Bitmap result = new Bitmap(source);

            sourceBitmap = result;

            // lock the source bitmap
            sourceBitmapData = getBitmapData(sourceBitmap, ref sourceWidth);
            sourcepBase = (Byte*)sourceBitmapData.Scan0.ToPointer();

            PixelData* pPixel;
            // get the top left hand pixel for the transparent color
            PixelData transparentPixel = *((PixelData*)(sourcepBase));
            *sourcepBase = 0xff;

            for (int y = 0; y < source.Height; y++)
            {
                pPixel = (PixelData*)(sourcepBase + y * sourceWidth);
                for (int x = 0; x < source.Width; x++)
                {
                    if (pPixel->red == 0 &&
                        pPixel->green == 0 &&
                        pPixel->blue == 0)
                    {
                        // if the mask pixel color is the trasparent mask, overwrite the 
                        // image pixel with our transparent colour
                        pPixel->red = transparentPixel.red;
                        pPixel->blue = transparentPixel.blue;
                        pPixel->green = transparentPixel.green;
                    }
                    pPixel++;
                }
            }

            sourceBitmap.UnlockBits(sourceBitmapData);
            sourceBitmapData = null;
            sourcepBase = null;

            return result;
        }

        /// <summary>
        /// Image process an image and apply the mask to crop and set up transparancy.
        /// </summary>
        /// <param name="source">source image to work on. This image is changed by the operation.
        /// </param>
        /// <param name="mask">mask image to be applied.</param>
        /// <param name="sourceRectangle">rectangle within the source which is to be masked with
        /// the mask.</param>
        /// <param name="drawTransparentColor">If this is true a much simpler masking is
        /// performed, where the maks is simply drawn using a transparent color over the
        /// top of the source. This is used when the masking process is implementing the
        /// cursor.</param>
        /// <returns>the processed bitmap</returns>
        public static Bitmap ProcessMask(Bitmap source, Bitmap mask, Rectangle sourceRectangle, bool drawTransparentColor)
        {
            if (drawTransparentColor)
            {
                sourceBitmap = source;
                maskBitmap = mask;
                maskRect = sourceRectangle;

                sourceBitmapData = getBitmapData(sourceBitmap, ref sourceWidth);
                sourcepBase = (Byte*)sourceBitmapData.Scan0.ToPointer();

                maskBitmapData = getBitmapData(maskBitmap, ref maskWidth);
                maskpBase = (Byte*)maskBitmapData.Scan0.ToPointer();

                float maskXStep = (float)maskBitmap.Width / (float)sourceRectangle.Width;
                float maskYStep = (float)maskBitmap.Height / (float)sourceRectangle.Height;
                float maskX = 0;
                float maskY = 0;

                PixelData* mPixel;
                PixelData* pPixel;
                PixelData transparentPixel = *((PixelData*)(maskpBase));

                int value;

                for (int y = sourceRectangle.Top; y < sourceRectangle.Bottom; y++)
                {
                    pPixel = (PixelData*)(sourcepBase + y * sourceWidth + 
                        sourceRectangle.Left * sizeof(PixelData));
                    mPixel = (PixelData*)(maskpBase + (int)maskY * maskWidth);
                    for (int x = sourceRectangle.Left; x < sourceRectangle.Right; x++)
                    {
                        if (mPixel->red == transparentPixel.red &&
                            mPixel->green == transparentPixel.green &&
                            mPixel->blue == transparentPixel.blue)
                        {
                            // if the mask pixel color is the trasparent mask, overwrite the 
                            // image pixel with our transparent colour
                            pPixel->red = transparentPixel.red;
                            pPixel->blue = transparentPixel.blue;
                            pPixel->green = transparentPixel.green;
                        }
                        else
                        {
                            // add the mask colour onto the image colour
                            value = mPixel->red + pPixel->red;
                            if (value > 255) value = 255;
                            pPixel->red = (byte)value;
                            value = mPixel->green + pPixel->green;
                            if (value > 255) value = 255;
                            pPixel->green = (byte)value;
                            value = mPixel->blue + pPixel->blue;
                            if (value > 255) value = 255;
                            pPixel->blue = (byte)value;

                            // subtract the mask colour from the image colour
                            //value = pPixel->red - mPixel->red ;
                            //if (value < 0) value = 0;
                            //pPixel->red = (byte)value;
                            //value = pPixel->green - mPixel->green ;
                            //if (value < 0) value = 0;
                            //pPixel->green = (byte)value;
                            //value = pPixel->blue - mPixel->blue ;
                            //if (value < 0) value = 0;
                            //pPixel->blue = (byte)value;
                        }
                        int oldMaskX = (int)maskX;
                        maskX += maskXStep;
                        int dist = ((int)maskX) - oldMaskX;
                        mPixel += dist;
                        pPixel++;
                    }
                    maskX = 0;
                    maskY += maskYStep;
                }
                sourceBitmap.UnlockBits(sourceBitmapData);
                sourceBitmapData = null;
                sourcepBase = null;
                maskBitmap.UnlockBits(maskBitmapData);
                maskBitmapData = null;
                maskpBase = null;
                return sourceBitmap;
            }
            else
            {
                // simple draw
                ImageAttributes attrib = new ImageAttributes();
                attrib.SetColorKey(mask.GetPixel(0, 0), mask.GetPixel(0, 0));
                using (Graphics g = Graphics.FromImage(source))
                {
                    g.DrawImage(mask, sourceRectangle, 0, 0, mask.Width, mask.Height, GraphicsUnit.Pixel, attrib);
                }
                return source;
            }
        }

        /// <summary>
        /// Performs a color adjustment on the source bitmap.
        /// </summary>
        /// <param name="source">The bitmap to be processed. This is changed by the process
        /// action.</param>
        /// <param name="addChange">The colour containing the change values.</param>
        /// <returns>a reference to the updated bitmap.</returns>
        public static Bitmap ProcessColor(Bitmap source, Color addChange)
        {
            return ProcessColor(source, addChange.R, addChange.G, addChange.B);
        }

        /// <summary>
        /// Performs a color adjustment on the source bitmap. 
        /// </summary>
        /// <param name="source">The bitmap to be processed. This is changed by the
        /// process action.</param>
        /// <param name="redChange">change to the red value. Can be negative.</param>
        /// <param name="greenChange">change to the green value. Can be negative.</param>
        /// <param name="blueChange">change to the blue value. Can be negative.</param>
        /// <returns></returns>
        public static Bitmap ProcessColor(Bitmap source, int redChange, int greenChange, int blueChange)
        {
            sourceBitmap = source;

            // lock the source bitmap
            sourceBitmapData = getBitmapData(sourceBitmap, ref sourceWidth);
            sourcepBase = (Byte*)sourceBitmapData.Scan0.ToPointer();

            PixelData* pPixel;

            for (int y = 0; y < source.Height; y++)
            {
                pPixel = (PixelData*)(sourcepBase + y * sourceWidth);
                for (int x = 0; x < source.Width; x++)
                {
                    int redVal = pPixel->red + redChange;
                    if ( redVal <0 ) redVal = 0;
                    if ( redVal > 255) redVal = 255;
                    pPixel->red = (byte)redVal;

                    int greenVal = pPixel->green + greenChange;
                    if ( greenVal <0 ) greenVal = 0;
                    if ( greenVal > 255) greenVal = 255;
                    pPixel->green = (byte)greenVal;

                    int blueVal = pPixel->blue + blueChange;
                    if (blueVal < 0) blueVal = 0;
                    if (blueVal > 255) blueVal = 255;
                    pPixel->blue = (byte)blueVal;

                    pPixel++;
                }
            }

            sourceBitmap.UnlockBits(sourceBitmapData);
            sourceBitmapData = null;
            sourcepBase = null;

            return source;
        }

        /// <summary>
        /// Random generator. Seeded so that the behaviour is repeatable.
        /// </summary>
        public static Random rnd = new Random(1);

        /// <summary>
        /// Performs a "wild" process on the bitmap. A three random values are computed
        /// and then added to the red, green and blue components respectively. The values
        /// are wrapped around.
        /// </summary>
        /// <param name="source">Source bitmap. This is changed by the action.</param>
        /// <returns>updated bitmap.</returns>
        public static Bitmap wildProcessColor(Bitmap source)
        {
            sourceBitmap = source;

            int redChange = rnd.Next(255);
            int greenChange = rnd.Next(255);
            int blueChange = rnd.Next(255);
            
            // lock the source bitmap
            sourceBitmapData = getBitmapData(sourceBitmap, ref sourceWidth);
            sourcepBase = (Byte*)sourceBitmapData.Scan0.ToPointer();

            PixelData* pPixel;

            for (int y = 0; y < source.Height; y++)
            {
                pPixel = (PixelData*)(sourcepBase + y * sourceWidth);
                for (int x = 0; x < source.Width; x++)
                {
                    int redVal = (pPixel->red + redChange) % 256;
                    pPixel->red = (byte)redVal;

                    int greenVal = (pPixel->green + greenChange) % 256;
                    pPixel->green = (byte)greenVal;

                    int blueVal = (pPixel->blue + blueChange) % 256;
                    pPixel->blue = (byte)blueVal;

                    pPixel++;
                }
            }

            sourceBitmap.UnlockBits(sourceBitmapData);
            sourceBitmapData = null;
            sourcepBase = null;

            return source;
        }

        /// <summary>
        /// Adds noise to the bitmap. A given percentage of pixels are chosen and replaced
        /// by a pixel of random color.
        /// </summary>
        /// <param name="source">Source image. This image is changed by the operation.</param>
        /// <param name="noisePercentage">percentage of noise. The larger the number, the 
        /// more noise is added to the image.</param>
        /// <returns>the updated image.</returns>
        public static Bitmap AddNoiseProcessColor(Bitmap source, int noisePercentage)
        {
            sourceBitmap = source;

            // lock the source bitmap
            sourceBitmapData = getBitmapData(sourceBitmap, ref sourceWidth);
            sourcepBase = (Byte*)sourceBitmapData.Scan0.ToPointer();

            PixelData* pPixel;

            for (int y = 0; y < source.Height; y++)
            {
                pPixel = (PixelData*)(sourcepBase + y * sourceWidth);
                for (int x = 0; x < source.Width; x++)
                {
                    if (rnd.Next(100) < noisePercentage)
                    {
                        pPixel->red = (byte)rnd.Next(255);
                        pPixel->green = (byte)rnd.Next(255);
                        pPixel->blue = (byte)rnd.Next(255);
                    }

                    pPixel++;
                }
            }

            sourceBitmap.UnlockBits(sourceBitmapData);
            sourceBitmapData = null;
            sourcepBase = null;

            return source;
        }

        /// <summary>
        /// Helper method to get the raw bitmap data and compute the width.
        /// </summary>
        /// <param name="bitmap">Bitmap source.</param>
        /// <param name="width">Computed width.</param>
        /// <returns>The raw data from the bitmap.</returns>
        private static BitmapData getBitmapData(Bitmap bitmap, ref int width)
        {
            Rectangle bounds = new Rectangle(0, 0, bitmap.Width, bitmap.Height);

            // Calculate the number of bytes in a row
            // This is rounded up to be a multiple of 4, 
            // since a scan line in an image must always 
            // be a multiple of 4 bytes long. 
            
            width = bounds.Width * sizeof(PixelData);

            if (width % 4 != 0)
            {
                width += (4 - (width % 4));
            }

            return bitmap.LockBits(bounds, ImageLockMode.ReadWrite, PixelFormat.Format24bppRgb);
        }
    }
}
