using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace ImageProcess
{
    /// <summary>
    /// Performs the image processing for the backgound.
    /// </summary>
    public partial class MakeBackgroundForm : Form
    {
        /// <summary>
        /// Local copy of the source image.
        /// </summary>
        private Image sourceImage;

        /// <summary>
        /// Constructor. Sets the source image and begins the proces
        /// </summary>
        /// <param name="inImage">The image to be processed.</param>
        public MakeBackgroundForm(Image inImage)
        {
            sourceImage = inImage;
            setImage(inImage);
            InitializeComponent();
        }

        /// <summary>
        /// preview image, used to show the result of the effects.
        /// </summary>
        private Bitmap previewImage = null;
        /// <summary>
        /// clean copy, used to provide the revert to the original
        /// </summary>
        private Bitmap cleanCopyImage = null;
        /// <summary>
        /// Reference to the original image
        /// </summary>
        private Image originalImage = null;

        /// <summary>
        /// The result as picked up by the user of this class
        /// </summary>
        public Bitmap Result;

        /// <summary>
        /// Begin using a particular image for colorising.
        /// </summary>
        /// <param name="newImage">the image to be processed.</param>
        private void setImage(Image newImage)
        {
            if (newImage == null)
            {
                return;
            }

            originalImage = newImage;

            // make a clean copy of the original bitmap, the size of the display
            previewImage = new Bitmap(this.Width, this.Height);

            Rectangle destRect = new Rectangle(0,0,this.Width, this.Height);
            Rectangle srcRect = new Rectangle(0,0,originalImage.Width, originalImage.Height);

            using (Graphics g = Graphics.FromImage(previewImage))
            {
                g.DrawImage(originalImage, destRect, srcRect, GraphicsUnit.Pixel);
            }

            cleanCopyImage = new Bitmap(previewImage);

        }

        /// <summary>
        /// Just draws the background image.
        /// </summary>
        /// <param name="g"></param>
        private void doPaint(Graphics g)
        {
            g.DrawImage(previewImage,0,0);
        }

        /// <summary>
        /// Currently active edit item
        /// </summary>
        private imageProcessItem currentItem = null;

        #region Image Process methods

        private imageProcessItem dowildColorsProcessItem = null;
        private void dowildColors()
        {
            Cursor.Current = Cursors.WaitCursor;
            ProcessBitmaps.wildProcessColor(previewImage);
            if (dowildColorsProcessItem == null)
            {
                dowildColorsProcessItem = new imageProcessItem(dowildColors);
            }
            currentItem = dowildColorsProcessItem;
            Invalidate();
            Cursor.Current = Cursors.Default;
        }

        private imageProcessItem doAddNoiseProcessItem = null;

        private void doAddNoise()
        {
            Cursor.Current = Cursors.WaitCursor;
            // add 5 percent noise
            ProcessBitmaps.AddNoiseProcessColor(previewImage, 5);
            if (doAddNoiseProcessItem == null)
            {
                doAddNoiseProcessItem = new imageProcessItem(doAddNoise);
            }
            currentItem = doAddNoiseProcessItem;
            Invalidate();
            Cursor.Current = Cursors.Default;
        }

        private imageProcessItem doGreenTintItem = null;
        private void doGreenTint()
        {
            Cursor.Current = Cursors.WaitCursor;
            ProcessBitmaps.ProcessColor(previewImage, 0, 10, 0);
            if (doGreenTintItem == null)
            {
                doGreenTintItem = new imageProcessItem(doGreenTint);
            }
            currentItem = doGreenTintItem;
            Invalidate();
            Cursor.Current = Cursors.Default;
        }

        private imageProcessItem doRedTintItem = null;
        private void doRedTint()
        {
            Cursor.Current = Cursors.WaitCursor;
            ProcessBitmaps.ProcessColor(previewImage, 10, 0, 0);
            if (doRedTintItem == null)
            {
                doRedTintItem = new imageProcessItem(doRedTint);
            }
            currentItem = doRedTintItem;
            Invalidate();
            Cursor.Current = Cursors.Default;
        }

        private imageProcessItem doBlueTintItem = null;
        private void doBlueTint()
        {
            Cursor.Current = Cursors.WaitCursor;
            ProcessBitmaps.ProcessColor(previewImage, 0, 0, 10);
            if (doBlueTintItem == null)
            {
                doBlueTintItem = new imageProcessItem(doBlueTint);
            }
            currentItem = doBlueTintItem;
            Invalidate();
            Cursor.Current = Cursors.Default;
        }

        private imageProcessItem doYellowTintItem = null;
        private void doYellowTint()
        {
            Cursor.Current = Cursors.WaitCursor;
            ProcessBitmaps.ProcessColor(previewImage, 10, 10, 0);
            if (doYellowTintItem == null)
            {
                doYellowTintItem = new imageProcessItem(doYellowTint);
            }
            currentItem = doYellowTintItem;
            Invalidate();
            Cursor.Current = Cursors.Default;
        }

        private imageProcessItem doBrightenItem = null;
        private void doBrighten()
        {
            Cursor.Current = Cursors.WaitCursor;
            ProcessBitmaps.ProcessColor(previewImage, 10, 10, 10);
            if (doBrightenItem == null)
            {
                doBrightenItem = new imageProcessItem(doBrighten);
            }
            currentItem = doBrightenItem;
            Invalidate();
            Cursor.Current = Cursors.Default;
        }


        private imageProcessItem doDarkenItem = null;
        private void doDarken()
        {
            Cursor.Current = Cursors.WaitCursor;
            ProcessBitmaps.ProcessColor(previewImage, -10, -10, -10);
            if (doDarkenItem == null)
            {
                doDarkenItem = new imageProcessItem(doDarken);
            }
            currentItem = doDarkenItem;
            Invalidate();
            Cursor.Current = Cursors.Default;
        }

        private void doRevertToOriginal()
        {
            Cursor.Current = Cursors.WaitCursor;
            setImage(originalImage);
            Invalidate();
            Cursor.Current = Cursors.Default;
        }

        #endregion

        private void doKeyPressed(Keys key)
        {
            switch (key)
            {
                case Keys.Enter:
                    if (currentItem != null)
                    {
                        currentItem();
                    }
                    break;
            }
            this.Invalidate();
        }

        private void doClose()
        {
            Result = previewImage;
            Close();
        }

        protected override void OnPaintBackground(PaintEventArgs e)
        {
            // do nothing, we control the entire paint operation 
        }

        private void ColoriseImageForm_Paint(object sender, PaintEventArgs e)
        {
            doPaint(e.Graphics);
        }

        private void ColoriseImageForm_KeyDown(object sender, KeyEventArgs e)
        {
            doKeyPressed(e.KeyCode);
        }

        private void doneMenuItem_Click(object sender, EventArgs e)
        {
            doClose();
        }

        private void tintRedMenuItem_Click(object sender, EventArgs e)
        {
            doRedTint();
        }

        private void tintGreenMenuItem_Click(object sender, EventArgs e)
        {
            doGreenTint();
        }

        private void tintBlueMenuItem_Click(object sender, EventArgs e)
        {
            doBlueTint();
        }

        private void tintYellowMenuItem_Click(object sender, EventArgs e)
        {
            doYellowTint();
        }

        private void brightenMenuItem_Click(object sender, EventArgs e)
        {
            doBrighten();
        }

        private void darkenMenuItem_Click(object sender, EventArgs e)
        {
            doDarken();
        }

        private void revertMenuItem_Click(object sender, EventArgs e)
        {
            doRevertToOriginal();
        }

        private void wildMenuItem_Click(object sender, EventArgs e)
        {
            dowildColors();
        }

        private void addNoiseMenuItem_Click(object sender, EventArgs e)
        {
            doAddNoise();
        }
    }

    delegate void imageProcessItem () ;

}