using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Imaging;


namespace ImageProcess
{
    public partial class MakeSpriteForm : Form
    {
        enum ProcessState
        {
            Size,
            Move
        }

        private ProcessState stateValue;

        private ProcessState state
        {
            get
            {
                return stateValue;
            }
            set
            {
                stateValue = value;
                this.Text = "Crop : " + stateValue.ToString();
            }
        }

        private int targetWidth;
        public Bitmap Result;

        public MakeSpriteForm( Image inImage, int inTargetWidth )
        {
            InitializeComponent();
            targetWidth = inTargetWidth;
            Tag = inImage;
        }

        private void doClose()
        {
            Cursor.Current = Cursors.WaitCursor;

            // make a rectangle the size of the final destination
            Rectangle outRect = new Rectangle();
            outRect.X = 0;
            outRect.Y = 0;
            outRect.Width = targetWidth;
            outRect.Height = (int)
                ((float)targetWidth * ((float)maskRect.Height / (float)maskRect.Width));

            Bitmap finalBitmap = new Bitmap(outRect.Width, outRect.Height);

            // draw the cropped part of the final image onto here
            using (Graphics g = Graphics.FromImage(finalBitmap))
            {
                g.DrawImage(cleanScreenCopy, outRect, maskRect, GraphicsUnit.Pixel);
            }

            // crop the image and perform the transparency 
            ProcessBitmaps.ProcessMask((Bitmap)finalBitmap, (Bitmap)imageCropMask, outRect, true);
            Result = finalBitmap;
            Close();
            Cursor.Current = Cursors.Default;
        }

        private Image originalImage;
        private Image screenBuffer;
        private Image cleanScreenCopy;

        private bool clearScreen = true;

        private Rectangle srcRect = new Rectangle(0,0,0,0);
        private Rectangle viewRect = new Rectangle(0, 0, 0,0);
        private Rectangle maskRect = new Rectangle(0, 0, 0, 0);
        private Rectangle oldMaskRect = new Rectangle(0, 0, 0, 0);

        private int moveStepSize = 10;
        private int sizeStepSize = 6;
        private int halfSizeStepSize = 3;
        private int viewX;
        private int cropMaxX;
        private int viewY;
        private int cropMaxY;

        private string currentMaskFilenameValue = null;
        private Image imageCropMask = null;
        private Image imageDrawMask = null;

        private string currentMaskFilename {
            set
            {
                if (value == currentMaskFilename)
                {
                    return;
                }
                imageCropMask = getBitmapResource(value);
                if (imageCropMask == null)
                {
                    return;
                }
                Cursor.Current = Cursors.WaitCursor;
                // make the new mask into an outline for drawing
                imageDrawMask = ProcessBitmaps.MakeBlackTransparent((Bitmap) imageCropMask);
                currentMaskFilenameValue = value;
                clearScreen = true;
                // update the cursor display to show the new shape
                removeMaskCursor();
                updateMaskCursor();
                Invalidate();
                Cursor.Current = Cursors.Default;
            }
            get
            {
                return currentMaskFilenameValue;
            }
        }

        protected override void OnPaintBackground(PaintEventArgs e)
        {
            // do nothing, we control the entire paint operation 
        }

        private void doPaint(Graphics g)
        {
            if (clearScreen)
            {
                g.Clear(Color.White);
                clearScreen = false;
            }
            g.DrawImage( screenBuffer, viewX, viewY);
        }

        private void removeMaskCursor()
        {
            using (Graphics g = Graphics.FromImage(screenBuffer))
            {
                // clean the old version away
                g.DrawImage(cleanScreenCopy, maskRect, maskRect, GraphicsUnit.Pixel);
            }
        }

        private void updateMaskCursor()
        {
            clipWithMask(screenBuffer, imageDrawMask, false);
            this.Invalidate();
        }

        /// <summary>
        /// Currently executing assembly. Cached because it is used
        /// to load a lot of resources. 
        /// </summary>
        readonly System.Reflection.Assembly assembly =
            System.Reflection.Assembly.GetExecutingAssembly();

        /// <summary>
        /// Returns a bitmap loaded from the resource.
        /// </summary>
        /// <param name="name">The name of the resource. The path to the
        /// resource is not required</param>
        /// <returns>the bitmap as found</returns>
        private Bitmap getBitmapResource(string name)
        {
            System.IO.Stream bitmapStream = assembly.GetManifestResourceStream(name);
            if (bitmapStream == null)
            {
                System.Windows.Forms.MessageBox.Show("Missing file : " + name, "Bitmap Load");
                return null;
            }
            return new Bitmap(bitmapStream);
        }

        private void clipWithMask(
            Image sourceImage,
            Image imageCropMask,
            bool drawTransparentColor)
        {
            
            if (imageCropMask == null)
            {
                return;
            }

            Rectangle clipRect = maskRect;
            clipRect.X -= viewX;
            clipRect.Y -= viewY;

            ProcessBitmaps.ProcessMask((Bitmap)sourceImage, (Bitmap)imageCropMask, maskRect, drawTransparentColor);

            Image result = new Bitmap(maskRect.Width, maskRect.Height);
            Rectangle resultRect = new Rectangle(0, 0, maskRect.Width, maskRect.Height);

            using (Graphics g = Graphics.FromImage(result))
            {
                g.DrawImage(sourceImage, resultRect, maskRect, GraphicsUnit.Pixel);
            }
        }

        private void moveCropCursor(Keys key)
        {
            removeMaskCursor();
            switch (key)
            {
                case Keys.Left:
                    maskRect.X -= moveStepSize;
                    break;
                case Keys.Right:
                    maskRect.X += moveStepSize;
                    break;
                case Keys.Up:
                    maskRect.Y -= moveStepSize;
                    break;
                case Keys.Down:
                    maskRect.Y += moveStepSize;
                    break;
                case Keys.Enter:
                    state = ProcessState.Size;
                    break;
            }
            updateMaskCursor();
        }

        private void sizeCropCursor(Keys key)
        {
            removeMaskCursor();
            switch (key)
            {
                case Keys.Right:
                    maskRect.Width += sizeStepSize;
                    break;
                case Keys.Up:
                    maskRect.Height -= sizeStepSize;
                    break;
                case Keys.Down:
                    maskRect.Height += sizeStepSize;
                    break;
                case Keys.Left:
                    maskRect.Width -= sizeStepSize;
                    break;
                case Keys.Enter:
                    state = ProcessState.Move;
                    break;
            }
            updateMaskCursor();
        }

        private void keyDown(Keys e)
        {
            switch (state)
            {
                case ProcessState.Move:
                    moveCropCursor(e);
                    break;
                case ProcessState.Size:
                    sizeCropCursor(e);
                    break;
            }
            this.Invalidate();
        }

        private void startCrop(Image newImage)
        {
            if (newImage == null)
            {
                return;
            }

            originalImage = newImage;

            float aspect = (float)originalImage.Width / (float)originalImage.Height;

            if (aspect > 1)
            {
                // wider than high
                viewRect.Width = this.Width;
                viewRect.Height = (int)((float)this.Width / aspect);
                viewX = 0;
                viewY = (this.Height - viewRect.Height) / 2;
            }
            else
            {
                // higher than wide
                viewRect.Height = this.Height;
                viewRect.Width = (int)((float)this.Height * aspect);
                viewX = (this.Width - viewRect.Width) / 2;
                viewY = 0;
            }

            srcRect.X = 0;
            srcRect.Y = 0;
            srcRect.Width = originalImage.Width;
            srcRect.Height = originalImage.Height;

            // make a clean copy of the original bitmap, the size of the display
            cleanScreenCopy = new Bitmap(viewRect.Width, viewRect.Height);
            using (Graphics g = Graphics.FromImage(cleanScreenCopy))
            {
                g.DrawImage(newImage, viewRect, srcRect, GraphicsUnit.Pixel);
            }

            // now copy the clean bitmap into the display bitmap

            screenBuffer = new Bitmap(cleanScreenCopy);

            cropMaxX = viewX + viewRect.Width;
            cropMaxY = viewY + viewRect.Height;

            maskRect.X = viewX + (viewRect.Width / 2) - halfSizeStepSize;
            maskRect.Y = viewY + (viewRect.Height / 2) - halfSizeStepSize;
            maskRect.Width = 50;
            maskRect.Height = 50;

            currentMaskFilename = "ImageProcess.images.RoundWhiteEdged.png";
            state = ProcessState.Move;
            clearScreen = true;
            updateMaskCursor();
        }

        private void doneMenuItem_Click(object sender, EventArgs e)
        {
            doClose();
        }

        private void CropImage_Load(object sender, EventArgs e)
        {
            startCrop(Tag as Image);
        }

        private void CropImage_Paint(object sender, PaintEventArgs e)
        {
            doPaint(e.Graphics);
        }

        private void CropImage_KeyDown(object sender, KeyEventArgs e)
        {
            keyDown(e.KeyCode);
        }

        private void boxMenuItem_Click(object sender, EventArgs e)
        {
            currentMaskFilename = "ImageProcess.images.RectangleWhiteEdged.png";
        }

        private void circleMenuItem_Click(object sender, EventArgs e)
        {
            currentMaskFilename = "ImageProcess.images.RoundWhiteEdged.png";
        }

        private void starMenuItem_Click(object sender, EventArgs e)
        {
            currentMaskFilename = "ImageProcess.images.FivePointStarWhiteEdged.png";
        }

        private void heartMenuItem_Click(object sender, EventArgs e)
        {
            currentMaskFilename = "ImageProcess.images.HeartWhiteEdged.png";
        }

        private void sizeMenuItem_Click(object sender, EventArgs e)
        {
            state = ProcessState.Size;
        }

        private void moveMenuItem_Click(object sender, EventArgs e)
        {
            state = ProcessState.Move;
        }
    }
}