using System;
using System.Collections;
using System.Drawing;

namespace PlayWithFriends
{

	#region Script Execution Engine

	/// <summary>
	/// A script step. This contains a reference to the deletage being executed.
	/// Will be extended to form different kinds of sprite movement control.
	/// </summary>
	/// 

	public abstract class SpriteScriptStep
	{
		/// <summary>
		///  Performs the command for this step
		/// </summary>
		/// <param name="sprite">the sprite we are working on</param>
		abstract public void Command ( ScriptedSprite sprite ) ;

		#region Utility methods

		/// <summary>
		/// Moves on to the next step in the script.
		/// </summary>
		/// <param name="step">The current step.</param>
		/// <param name="SpriteScriptItems">The collection of steps to perform</param>
		/// <returns>next step, or same one if we are at the end.</returns>
		public static SpriteScriptStep NextStep ( 
			SpriteScriptStep step,
			ArrayList SpriteScriptItems ) 
		{
			int pos = SpriteScriptItems.IndexOf(step);
			pos++;
			if ( pos == SpriteScriptItems.Count ) 
			{
				// if we are at the end, just return the same one again
				return step;
			}
			return ( SpriteScriptItems[pos] as SpriteScriptStep );
		}

		/// <summary>
		/// Goes to the label supplied as a parameter.
		/// </summary>
		/// <param name="searchLabelText">text of the label to search for</param>
		/// <param name="SpriteScriptItems">the script to search in</param>
		/// <returns>the step reached, or a null if the label was not found.</returns>
		public static SpriteScriptStep GotoLabel ( 
			string searchLabelText,
			ArrayList SpriteScriptItems ) 
		{
			// look for the step with the appropriate label
			foreach ( SpriteScriptStep step in SpriteScriptItems ) 
			{
				if ( step is SpriteScriptStepLabel ) 
				{
					// got a label
					SpriteScriptStepLabel label = step as SpriteScriptStepLabel;
					if ( label.LabelText == searchLabelText ) 
					{
						return step;
					}
				}	
			}
			return null;
		}

		#endregion
	}

	#endregion

	#region Script moves

	#region Script control steps

	/// <summary>
	/// Pauses the script for the given number of ticks.
	/// </summary>
	public class SpriteScriptStepPause : SpriteScriptStep 
	{
		private int pauseLimit;

		/// <summary>
		/// Waits for the given number of ticks to pass.
		/// </summary>
		/// <param name="sprite">sprite sprite which is waiting</param>
		public override void Command ( ScriptedSprite sprite ) 
		{
			if ( sprite.Counter >= pauseLimit ) 
			{
				sprite.Counter = 0 ;
				// move on to the next step
				sprite.CurrentSpriteSciptStep = SpriteScriptStep.NextStep ( 
					sprite.CurrentSpriteSciptStep,
					sprite.CurrentAttackWave.ScriptItems );
			}
			else 
			{
				sprite.Counter++;
			}
		}

		/// <summary>
		/// Sets the pause value for this pause
		/// </summary>
		/// <param name="inPauseLimit">number of ticks for the script to pause</param>
		public SpriteScriptStepPause ( int inPauseLimit ) 
		{
			pauseLimit = inPauseLimit;
		}
	}

	/// <summary>
	/// Stops the script and kill the sprite being controlled. 
	/// </summary>
	public class SppriteScriptStepStop : SpriteScriptStep 
	{
		/// <summary>
		/// Does nothing, causing the script to end.
		/// </summary>
		/// <param name="sprite">sprite sprite we kill</param>
		public override void Command ( ScriptedSprite sprite ) 
		{
			sprite.IsAlive = false;
		}

		public SppriteScriptStepStop () 
		{
		}
	}

	/// <summary>
	/// Controls the visibility of the sprite
	/// </summary>
	public class SpriteScriptStepSetVisible : SpriteScriptStep 
	{
		private bool isVisible ;

		/// <summary>
		/// Sets the visibility of the sprite
		/// </summary>
		/// <param name="sprite">the sprite to place</param>
		public override void Command ( ScriptedSprite sprite ) 
		{
			sprite.Visible = isVisible;
			// move on to the next step
			sprite.CurrentSpriteSciptStep = SpriteScriptStep.NextStep ( 
				sprite.CurrentSpriteSciptStep,
				sprite.CurrentAttackWave.ScriptItems );
		}

		public SpriteScriptStepSetVisible ( bool inIsVisible ) 
		{
			isVisible = inIsVisible;
		}
	}

	#endregion

	#region Script branch steps

	/// <summary>
	/// Puts everone in the script at this place in the script. 
	/// </summary>
	public class SppriteScriptStepEveryoneHere : SpriteScriptStep 
	{
		/// <summary>
		/// Brings all the sprites to the same position in the script
		/// </summary>
		/// <param name="sprite">sprite sprite we kill</param>
		public override void Command ( ScriptedSprite sprite ) 
		{
			// move on to the next step
			sprite.CurrentSpriteSciptStep = SpriteScriptStep.NextStep ( 
				sprite.CurrentSpriteSciptStep,
				sprite.CurrentAttackWave.ScriptItems );

			// put everyone at this step
			foreach ( ScriptedSprite waveSprite in sprite.CurrentAttackWave.Sprites ) 
			{
				waveSprite.CurrentSpriteSciptStep = sprite.CurrentSpriteSciptStep;
			}
		}

		public SppriteScriptStepEveryoneHere () 
		{
		}
	}

	/// <summary>
	/// Allows a position in the sprite command sequence to be labeled.
	/// </summary>
	public class SpriteScriptStepLabel : SpriteScriptStep 
	{
		/// <summary>
		/// text label to uniquely idenfity this position in the script
		/// </summary>
		public string LabelText ;

		/// <summary>
		/// Does nothing, just move on to the next step
		/// </summary>
		/// <param name="sprite">has no effect on the sprite</param>
		public override void Command ( ScriptedSprite sprite ) 
		{
			// move on to the next step
			sprite.CurrentSpriteSciptStep = SpriteScriptStep.NextStep ( 
				sprite.CurrentSpriteSciptStep,
				sprite.CurrentAttackWave.ScriptItems );
		}

		public SpriteScriptStepLabel ( string inLabelText ) 
		{
			LabelText = inLabelText;
		}
	}

	/// <summary>
	/// Goto a given label in the sprite script
	/// </summary>
	public class SpriteScriptStepGoto : SpriteScriptStep 
	{
		/// <summary>
		/// text label we are going to
		/// </summary>
		public string LabelText ;

		/// <summary>
		/// Performs the goto. If the label cannot be found we go on to 
		/// the next step
		/// </summary>
		/// <param name="sprite">has no effect on the sprite</param>
		public override void Command ( ScriptedSprite sprite ) 
		{
			SpriteScriptStep dest = SpriteScriptStep.GotoLabel ( LabelText, sprite.CurrentAttackWave.ScriptItems ) ;

			if ( dest == null ) 
			{
				// move on to the next step - goto has failed.
				sprite.CurrentSpriteSciptStep = SpriteScriptStep.NextStep ( 
					sprite.CurrentSpriteSciptStep,
					sprite.CurrentAttackWave.ScriptItems );
			}
			else 
			{
				sprite.CurrentSpriteSciptStep = dest;
			}
		}

		public SpriteScriptStepGoto ( string inLabelText ) 
		{
			LabelText = inLabelText;
		}
	}

	#endregion

	#region Sprite position script steps

	/// <summary>
	/// Places the sprite at the given position
	/// </summary>
	public class SpriteScriptStepPlaceSprite : SpriteScriptStep 
	{
		private int x;
		private int y;

		/// <summary>
		/// Place the sprite at the requied position
		/// </summary>
		/// <param name="sprite">the sprite to place</param>
		public override void Command ( ScriptedSprite sprite ) 
		{
			sprite.X = x;
			sprite.Y = y;
			// move on to the next step
			sprite.CurrentSpriteSciptStep = SpriteScriptStep.NextStep ( 
				sprite.CurrentSpriteSciptStep,
				sprite.CurrentAttackWave.ScriptItems );
		}

		public SpriteScriptStepPlaceSprite ( int inX, int inY ) 
		{
			x = inX;
			y = inY;
		}
	}

	/// <summary>
	/// A sprite move step. Contains the distance to move and the number of steps
	/// to move in that distance. Also contains a member which is used to count 
	/// the steps.
	/// </summary>
	/// 

	public class SpriteScriptStepMove : SpriteScriptStep 
	{
		private float xchange;
		private float ychange;
		private int stepLimit;

		public override void Command ( ScriptedSprite sprite ) 
		{
			if ( !sprite.Visible  ) 
			{
				return;
			}

			if ( sprite.Counter == stepLimit ) 
			{
				// reached the end of execution of this step

				// reset the step count for the next time this step is used
				sprite.Counter = 0;

				// move on to the next step
				sprite.CurrentSpriteSciptStep = SpriteScriptStep.NextStep ( 
					sprite.CurrentSpriteSciptStep,
					sprite.CurrentAttackWave.ScriptItems );
			}
			else 
			{
				if ( sprite.Counter == 0 ) 
				{
					// set the initial floating point values to the position of the sprite
					sprite.Xfloat = sprite.X;
					sprite.Yfloat = sprite.Y;
				}

				// move the sprite
				sprite.Xfloat += xchange;
				sprite.Yfloat += ychange;
				sprite.X = (int) sprite.Xfloat;
				sprite.Y = (int) sprite.Yfloat;
				sprite.Counter++; 
			}
		}

		public SpriteScriptStepMove ( 
			float inXchange,
			float inYchange,
			int inStepLimit ) : base () 
		{
			xchange = inXchange;
			ychange = inYchange;
			stepLimit = inStepLimit;
		}

	}


	/// <summary>
	/// Moves from the present position of the sprite to the given location.
	/// Also contains a member which is used to count 
	/// the steps.
	/// </summary>
	/// 

	public class SpriteScriptStepMoveTo : SpriteScriptStep 
	{
		private float xchange;
		private float ychange;
		private float xdest;
		private float ydest ;

		private int stepLimit;

		public SpriteScriptStepMoveTo ( 
			float inXdest,
			float inYdest,
			int inStepLimit ) : base () 
		{
			xdest = inXdest;
			ydest = inYdest;
			stepLimit = inStepLimit;
		}

		public override void Command ( ScriptedSprite sprite ) 
		{
			if ( !sprite.Visible  ) 
			{
				return;
			}

			if ( sprite.Counter == stepLimit ) 
			{
				// reached the end of execution of this step

				// reset the step count for the next time this step is used
				sprite.Counter = 0;

				// move on to the next step
				sprite.CurrentSpriteSciptStep = SpriteScriptStep.NextStep ( 
					sprite.CurrentSpriteSciptStep,
					sprite.CurrentAttackWave.ScriptItems );
			}
			else 
			{
				if ( sprite.Counter == 0 ) 
				{
					// set the initial floating point values to the position of the sprite
					sprite.Xfloat = sprite.X;
					sprite.Yfloat = sprite.Y;

					// calculate the steps to be moved
					xchange = (xdest - sprite.Xfloat) / stepLimit ;
					ychange = (ydest - sprite.Yfloat) / stepLimit ;
				}

				// move the sprite
				sprite.Xfloat += xchange;
				sprite.Yfloat += ychange;
				sprite.X = (int) sprite.Xfloat;
				sprite.Y = (int) sprite.Yfloat;
				sprite.Counter++; 
			}
		}
	}


	/// <summary>
	/// Chase the ship. Contains the distance to move and the number of steps
	/// we should chase for. Also contains a member which is used to count 
	/// the steps.
	/// </summary>
	/// 

	public class SpriteScriptStepChase : SpriteScriptStep 
	{
		private float speed;
		private int stepLimit;

		public SpriteScriptStepChase ( float inSpeed, int inStepLimit ) : base () 
		{
			speed = inSpeed;
			stepLimit = inStepLimit;
		}

		public override void Command ( ScriptedSprite sprite ) 
		{
			if ( !sprite.Visible  ) 
			{
				return;
			}

			if ( sprite.Counter == stepLimit ) 
			{
				// reached the end of execution of this step

				// reset the step count for the next time this step is used
				sprite.Counter = 0;

				// move on to the next step
				sprite.CurrentSpriteSciptStep = SpriteScriptStep.NextStep ( 
					sprite.CurrentSpriteSciptStep,
					sprite.CurrentAttackWave.ScriptItems );
			}
			else 
			{
				if ( sprite.Counter == 0 ) 
				{
					// set the initial floating point values to the position of the sprite
					sprite.Xfloat = sprite.X;
					sprite.Yfloat = sprite.Y;
				}

				// move the sprite
				float dx = sprite.Xfloat - ScriptedSprite.ship.X;
				float dy = sprite.Yfloat - ScriptedSprite.ship.Y;

				if ( dx > 0 ) 
				{
					sprite.Xfloat -= speed;
				}
				if ( dx < 0 ) 
				{
					sprite.Xfloat += speed;
				}

				if ( dy > 0 ) 
				{
					sprite.Yfloat -= speed;
				}
				if ( dy < 0 ) 
				{
					sprite.Yfloat += speed;
				}

				sprite.X = (int) sprite.Xfloat;
				sprite.Y = (int) sprite.Yfloat;
				sprite.Counter++; 
			}
		}
	}

	#endregion

	#endregion

	#region Attack wave management

	/// <summary>
	/// The movement script and sprites for an attack wave.
	/// </summary>
	public class AttackWave 
	{
		/// <summary>
		/// The script controlling this attack wave
		/// </summary>
		public ArrayList ScriptItems = new ArrayList();

		/// <summary>
		/// The sprites in this attack wave
		/// </summary>
		public ArrayList Sprites = new ArrayList();

		/// <summary>
		/// Number of sprites in the wave (actually the length of the ArrayList)
		/// </summary>
		public int NoOfSprites;

		/// <summary>
		/// Store gained by killing each of the sprites.
		/// </summary>
		public int SpriteValue;

		/// <summary>
		/// Builds a list of introduction moves. Each of these is a pause of a particular length.
		/// This method is used to introduce a line of attackers which all follow the same sequence
		/// 
		/// </summary>
		/// <param name="moves">Attackwave to add the moves to</param>
		/// <param name="noOfSprites">number of sprites in the introduction sequence</param>
		/// <param name="spriteGap">the gap in steps between each sprite</param>
		public static void BuildIntroList ( AttackWave moves, int noOfSprites, int spriteGap ) 
		{
			for ( int i=0 ; i < noOfSprites ; i++ ) 
			{
				moves.ScriptItems.Add ( new SpriteScriptStepPause ( spriteGap ) );
			}
		}

		/// <summary>
		/// Builds an empty attack wave. Constructs the sprites used in this wave from the
		/// parameters given.
		/// </summary>
		/// <param name="inNoOfSprites">the number of sprites in the wave.</param>
		/// <param name="inSpriteValue">the value scored by each sprite.</param>
		/// <param name="bitmap">the design of this sprite. Is an animated design and
		/// will be converted into an AnimatedBitmap by the sprite.</param>
		/// <param name="explodeBitmap">Animated explosion sequence for the sprite</param>
		/// <param name="hitSound">sound made by the sprite when it is hit.</param>
		/// <param name="explodeSound">sound made by the sprite when it is destroyed.</param>
		/// <param name="boundingRectangle">bounding rectangle for the sprite</param>
		
		public AttackWave ( 
			int inNoOfSprites, 
			int inSpriteValue,
			Bitmap bitmap, 
			Bitmap explodeBitmap,
			Sound hitSound,
			Sound explodeSound,
			Rectangle boundingRectangle
			) 
		{
			NoOfSprites = inNoOfSprites;
			SpriteValue = inSpriteValue;
			for ( int i = 0 ; i < NoOfSprites ; i++ ) 
			{
				Sprites.Add ( new AnimatedAlienSprite ( 
					bitmap,
					explodeBitmap,
					hitSound,
					explodeSound,
					boundingRectangle )
				);
			}
		}

        /// <summary>
        /// Builds a boss attack wave. The boss is a single, non-animated bitmap
        /// </summary>
        /// <param name="inSpriteValue">value of the boss</param>
        /// <param name="bitmap">bitmap image</param>
        /// <param name="explodeBitmap">explode bitmap</param>
        /// <param name="hitSound">sound that it makes when it is hit</param>
        /// <param name="explodeSound">sound made when it explodes</param>
        /// <param name="boundingRectangle">bounding rectangle for sprite</param>
        public AttackWave(
            int inSpriteValue,
            Bitmap bitmap,
            Bitmap explodeBitmap,
            Sound hitSound,
            Sound explodeSound,
            Rectangle boundingRectangle
            )
        {
            NoOfSprites = 1;
            SpriteValue = inSpriteValue;
            Sprites.Add(new ImageAlienSprite(
                bitmap,
                explodeBitmap,
                hitSound,
                explodeSound,
                boundingRectangle)
            );
        }

        /// <summary>
		/// Checks to see if the missile given has hit any of the sprites in the 
		/// attack wave.
		/// </summary>
		/// <param name="missile">missile to check with</param>
		/// <returns>the score achieved by the hit, 0 if no sprite was hit.</returns>
		public int CheckForHitScore ( MissileSprite missile ) 
		{
			foreach ( AlienSprite alien in Sprites ) 
			{
                if (alien.State == AlienSpriteStates.visible) 
				{
					if ( missile.SpriteRectangle.IntersectsWith(alien.SpriteRectangle)) 
					{
						if ( alien.HitHasKilled( missile.MissilePower ) ) 
						{
							return SpriteValue;
						}
						missile.State = MissileSpriteStates.loaded;
						return 0;
					}
				}
			}
			return 0;
		}

		/// <summary>
		/// Checks to see if the ship has hit any of the sprites in the attack 
		/// wave.
		/// </summary>
		/// <param name="ship">ship to check for</param>
		/// <returns>true if the ship was hit</returns>
		public bool CheckForShipCollide ( ShipSprite ship ) 
		{
			foreach ( AlienSprite alien in Sprites ) 
			{
				if ( alien.State == AlienSpriteStates.visible ) 
				{
					if ( ship.SpriteRectangle.IntersectsWith(alien.SpriteRectangle)) 
					{
						ship.HitHasKilled ( 1 ) ;
						alien.HitHasKilled ( 50 ) ;
						return true;
					}
				}
			}
			return false;
		}

		/// <summary>
		/// Resets the wave. Asks each sprite in the wave to reset itself.
		/// </summary>
		public void ResetWave () 
		{
			for ( int i=0 ; i < NoOfSprites ; i++ ) 
			{
				ScriptedSprite sprite = Sprites[i] as ScriptedSprite;
				sprite.Reset();
				sprite.CurrentAttackWave = this;
			}
		}

		/// <summary>
		/// Checks if all the sprites in the wave have set the IsAlive
		/// flag to false. If they have the level is over.
		/// </summary>
		/// <returns>true if the level is over</returns>
		public bool WaveDestroyed () 
		{
			foreach ( ScriptedSprite sprite in Sprites ) 
			{
				if ( sprite.IsAlive ) 
				{
					return false;
				}
			}
			return true;
		}

		// using an idealised coordinate set for the move designs. The screen is 100 
		// pixels wide and 100 high.

		/// <summary>
		/// X factor to multiply widths in scripts to get the actual X value on the screen.
		/// The value must have been set by the method setStatics
		/// </summary>
		static private float Xf;

		/// <summary>
		/// Y factor to multiply widths in scripts to get the actual X value on the screen.
		/// The value must have been set by the method setStatics
		/// </summary>
		static private float Yf;

		/// <summary>
		/// Width of the sprites in the wave. Set by the method setStatics
		/// </summary>
		static private int spriteWidth;

		/// <summary>
		/// Height of the sprites in the wave. Set by the method setStatics
		/// </summary>
		static private int spriteHeight;

		/// <summary>
		/// Sets the width and height factors for sprite position calculations.
		/// </summary>
		/// <param name="rect">size of the rectangle the sprite will be moving in</param>
		/// <param name="sprite">sprite to read the sizes from</param>
		public static void setStatics ( Rectangle rect, ScriptedSprite sprite ) 
		{
			spriteWidth = sprite.SpriteRectangle.Width;
			spriteHeight = sprite.SpriteRectangle.Height;
			Xf = (rect.Width - spriteWidth)  / 100.0f;
			Yf = (rect.Height - spriteHeight) / 100.0f;
		}

		/// <summary>
		/// An attack wave of sprites which move around the edge of the screen clockwise.
		/// </summary>
		/// <param name="noOfSprites">number of sprites in the wave</param>
		/// <param name="spriteValue">number of points for each sprite</param>
		/// <param name="spriteGap">gap in steps between each sprite in the wave</param>
		/// <param name="bitmap">bitmap for the sprite</param>
		/// <param name="explodeBitmap">bitmap for the sprite explosion</param>
		/// <param name="hitSound">sound made when a sprite is hit</param>
		/// <param name="explodeSound">sound made when a sprite explodes</param>
		/// <param name="boundingRectangle">bounding rectangle for this wave</param>
		/// <returns>the attackwave produced</returns>
		public static AttackWave ClockSquareDance ( 
			int noOfSprites, 
			int spriteValue,
			int spriteGap,
			Bitmap bitmap, 
			Bitmap explodeBitmap,
			Sound hitSound,
			Sound explodeSound,
			Rectangle boundingRectangle ) 
		{
			AttackWave result = new AttackWave (
				noOfSprites,
				spriteValue,
				bitmap,
				explodeBitmap,
				hitSound,
				explodeSound,
				boundingRectangle );

			setStatics ( boundingRectangle, result.Sprites[0] as ScriptedSprite );

			AttackWave.BuildIntroList ( result, result.NoOfSprites, spriteGap );

			int pos=0;

			foreach ( ScriptedSprite sprite in result.Sprites ) {
				sprite.ResetSpriteScriptStep = result.ScriptItems[pos] as SpriteScriptStep;
				sprite.ResetSpritePositionX = -100;
				sprite.ResetSpritePositionY = -100;
				pos++;
			}

			result.ScriptItems.Add ( new SpriteScriptStepSetVisible ( true ) );
			result.ScriptItems.Add ( new SpriteScriptStepMove( 2, 2, 50) );
			result.ScriptItems.Add ( new SpriteScriptStepLabel("restart") );
			result.ScriptItems.Add ( new SpriteScriptStepMove( 2*Xf, 0, 50) );
			result.ScriptItems.Add ( new SpriteScriptStepMove( 0, 2*Yf, 50) );
			result.ScriptItems.Add ( new SpriteScriptStepMove( -2*Xf, 0, 50) );
			result.ScriptItems.Add ( new SpriteScriptStepMove( 0, -2*Yf, 50) );
			result.ScriptItems.Add ( new SpriteScriptStepGoto ("restart") );

			result.ResetWave();

			return result;
		}

		/// <summary>
		/// An attack wave of sprites which move around the edge of the screen clockwise for 
		/// one loop and then attack the ship.
		/// </summary>
		/// <param name="noOfSprites">number of sprites in the wave</param>
		/// <param name="spriteValue">number of points for each sprite</param>
		/// <param name="spriteGap">gap in steps between each sprite in the wave</param>
		/// <param name="bitmap">bitmap for the sprite</param>
		/// <param name="explodeBitmap">bitmap for the sprite explosion</param>
		/// <param name="hitSound">sound made when a sprite is hit</param>
		/// <param name="explodeSound">sound made when a sprite explodes</param>
		/// <param name="boundingRectangle">bounding rectangle for this wave</param>
		/// <returns>the attackwave produced</returns>

		public static AttackWave ClockSquareDanceAttack ( 
			int noOfSprites, 
			int spriteValue,
			int spriteGap,
			Bitmap bitmap, 
			Bitmap explodeBitmap,
			Sound hitSound,
			Sound explodeSound,
			Rectangle boundingRectangle ) 
		{
			AttackWave result = new AttackWave (
				noOfSprites,
				spriteValue,
				bitmap,
				explodeBitmap,
				hitSound,
				explodeSound,
				boundingRectangle );

			setStatics ( boundingRectangle, result.Sprites[0] as ScriptedSprite );

			AttackWave.BuildIntroList ( result, result.NoOfSprites, spriteGap );

			int pos=0;

			foreach ( ScriptedSprite sprite in result.Sprites ) 
			{
				sprite.ResetSpriteScriptStep = result.ScriptItems[pos] as SpriteScriptStep;
				sprite.ResetSpritePositionX = -100;
				sprite.ResetSpritePositionY = -100;
				pos++;
			}

			result.ScriptItems.Add ( new SpriteScriptStepSetVisible ( true ) );
			result.ScriptItems.Add ( new SpriteScriptStepMove( 2, 2, 50) );
			result.ScriptItems.Add ( new SpriteScriptStepMove( 2*Xf, 0, 50) );
			result.ScriptItems.Add ( new SpriteScriptStepMove( 0, 2*Yf, 50) );
			result.ScriptItems.Add ( new SpriteScriptStepMove( -2*Xf, 0, 50) );
			result.ScriptItems.Add ( new SpriteScriptStepMove( 0, -2*Yf, 50) );
			result.ScriptItems.Add ( new SpriteScriptStepChase ( 2, 150 ) );
			result.ScriptItems.Add ( new SpriteScriptStepMove( -2*Xf, -2*Yf, 50) );
			result.ScriptItems.Add ( new SppriteScriptStepStop() );

			result.ResetWave();

			return result;
		}

		/// <summary>
		/// An attack wave of sprites which move around the edge of the screen anticlockwise.
		/// </summary>
		/// <param name="noOfSprites">number of sprites in the wave</param>
		/// <param name="spriteValue">number of points for each sprite</param>
		/// <param name="spriteGap">gap in steps between each sprite in the wave</param>
		/// <param name="bitmap">bitmap for the sprite</param>
		/// <param name="explodeBitmap">bitmap for the sprite explosion</param>
		/// <param name="hitSound">sound made when a sprite is hit</param>
		/// <param name="explodeSound">sound made when a sprite explodes</param>
		/// <param name="boundingRectangle">bounding rectangle for this wave</param>
		/// <returns>the attackwave produced</returns>
		public static AttackWave AntiClockSquareDance ( 
			int noOfSprites, 
			int spriteValue,
			int spriteGap,
			Bitmap bitmap, 
			Bitmap explodeBitmap,
			Sound hitSound,
			Sound explodeSound,
			Rectangle boundingRectangle ) 
		{
			AttackWave result = new AttackWave (
				noOfSprites,
				spriteValue,
				bitmap,
				explodeBitmap,
				hitSound,
				explodeSound,
				boundingRectangle );

			setStatics ( boundingRectangle, result.Sprites[0] as ScriptedSprite );

			AttackWave.BuildIntroList ( result, result.NoOfSprites, spriteGap );

			int pos=0;

			foreach ( ScriptedSprite sprite in result.Sprites ) 
			{
				sprite.ResetSpriteScriptStep = result.ScriptItems[pos] as SpriteScriptStep;
				sprite.ResetSpritePositionX = (int)(200*Xf) ;
				sprite.ResetSpritePositionY = (int)(-100*Yf);
				pos++;
			}

			result.ScriptItems.Add ( new SpriteScriptStepSetVisible ( true ) );
			result.ScriptItems.Add ( new SpriteScriptStepMove( -2*Xf, 2*Yf, 50) );
			result.ScriptItems.Add ( new SpriteScriptStepLabel("restart") );
			result.ScriptItems.Add ( new SpriteScriptStepMove( -2*Xf, 0, 50) );
			result.ScriptItems.Add ( new SpriteScriptStepMove( 0, 2*Yf, 50) );
			result.ScriptItems.Add ( new SpriteScriptStepMove( 2*Xf, 0, 50) );
			result.ScriptItems.Add ( new SpriteScriptStepMove( 0, -2*Yf, 50) );
			result.ScriptItems.Add ( new SpriteScriptStepGoto ("restart") );

			result.ResetWave();

			return result;
		}

		/// <summary>
		/// An attack wave of sprites which move around the edge of the screen anticlockwise. 
		/// At the end of one loop all the sprites turn on the ship at once.
		/// </summary>
		/// <param name="noOfSprites">number of sprites in the wave</param>
		/// <param name="spriteValue">number of points for each sprite</param>
		/// <param name="spriteGap">gap in steps between each sprite in the wave</param>
		/// <param name="bitmap">bitmap for the sprite</param>
		/// <param name="explodeBitmap">bitmap for the sprite explosion</param>
		/// <param name="hitSound">sound made when a sprite is hit</param>
		/// <param name="explodeSound">sound made when a sprite explodes</param>
		/// <param name="boundingRectangle">bounding rectangle for this wave</param>
		/// <returns>the attackwave produced</returns>
		public static AttackWave AntiClockSquareDanceMobAttack ( 
			int noOfSprites, 
			int spriteValue,
			int spriteGap,
			Bitmap bitmap, 
			Bitmap explodeBitmap,
			Sound hitSound,
			Sound explodeSound,
			Rectangle boundingRectangle ) 
		{
			AttackWave result = new AttackWave (
				noOfSprites,
				spriteValue,
				bitmap,
				explodeBitmap,
				hitSound,
				explodeSound,
				boundingRectangle );

			setStatics ( boundingRectangle, result.Sprites[0] as ScriptedSprite );

			AttackWave.BuildIntroList ( result, result.NoOfSprites, spriteGap );

			int pos=0;

			foreach ( ScriptedSprite sprite in result.Sprites ) 
			{
				sprite.ResetSpriteScriptStep = result.ScriptItems[pos] as SpriteScriptStep;
				sprite.ResetSpritePositionX = (int)(200*Xf) ;
				sprite.ResetSpritePositionY = (int)(-100*Yf);
				pos++;
			}

			result.ScriptItems.Add ( new SpriteScriptStepSetVisible ( true ) );
			result.ScriptItems.Add ( new SpriteScriptStepMove( -2*Xf, 2*Yf, 50) );
			result.ScriptItems.Add ( new SpriteScriptStepMove( -2*Xf, 0, 50) );
			result.ScriptItems.Add ( new SpriteScriptStepMove( 0, 2*Yf, 50) );
			result.ScriptItems.Add ( new SpriteScriptStepMove( 2*Xf, 0, 50) );
			result.ScriptItems.Add ( new SpriteScriptStepMove( 0, -2*Yf, 20) );
			result.ScriptItems.Add ( new SppriteScriptStepEveryoneHere() );
			result.ScriptItems.Add ( new SpriteScriptStepChase ( 1, 10 ) );
			result.ScriptItems.Add ( new SpriteScriptStepChase ( 2, 20 ) );
			result.ScriptItems.Add ( new SpriteScriptStepChase ( 3, 20 ) );
			result.ScriptItems.Add ( new SpriteScriptStepChase ( 4, 30 ) );
			result.ScriptItems.Add ( new SpriteScriptStepMove( -2*Xf, -2*Yf, 50) );
			
			result.ResetWave();

			return result;
		}

		/// <summary>
		/// An attack wave of sprites which move down the screen at speed. 
		/// </summary>
		/// <param name="noOfSprites">number of sprites in the wave</param>
		/// <param name="spriteValue">number of points for each sprite</param>
		/// <param name="spriteGap">gap in steps between each sprite in the wave</param>
		/// <param name="bitmap">bitmap for the sprite</param>
		/// <param name="explodeBitmap">bitmap for the sprite explosion</param>
		/// <param name="hitSound">sound made when a sprite is hit</param>
		/// <param name="explodeSound">sound made when a sprite explodes</param>
		/// <param name="boundingRectangle">bounding rectangle for this wave</param>
		/// <returns>the attackwave produced</returns>
		public static AttackWave DiveBombFlat ( 
			int noOfSprites, 
			int spriteValue,
			int spriteGap,
			Bitmap bitmap, 
			Bitmap explodeBitmap,
			Sound hitSound,
			Sound explodeSound,
			Rectangle boundingRectangle ) 
		{
			AttackWave result = new AttackWave (
				noOfSprites,
				spriteValue,
				bitmap,
				explodeBitmap,
				hitSound,
				explodeSound,
				boundingRectangle );

			setStatics ( boundingRectangle, result.Sprites[0] as ScriptedSprite );

			int pos = 0;
			float gap = ( 100 / (float) result.NoOfSprites ) * Xf;

			result.ScriptItems.Add ( new SpriteScriptStepSetVisible ( true ) );
			result.ScriptItems.Add ( new SpriteScriptStepMove( 0, 4 * Yf, 25) );
			result.ScriptItems.Add ( new SppriteScriptStepStop() );

			foreach ( ScriptedSprite sprite in result.Sprites ) 
			{
				sprite.ResetSpriteScriptStep = result.ScriptItems[0] as SpriteScriptStep ;
				sprite.ResetSpritePositionX = (int) ( (pos * gap) + (spriteWidth/2) ) ;
				sprite.ResetSpritePositionY = -spriteHeight ;
				pos++;
			}

			result.ResetWave();

			return result;
		}

		/// <summary>
		/// An attack wave of sprites which move down the screen at speed, the sprites
		/// are in a diagonal with the left side low. 
		/// </summary>
		/// <param name="noOfSprites">number of sprites in the wave</param>
		/// <param name="spriteValue">number of points for each sprite</param>
		/// <param name="spriteGap">gap in steps between each sprite in the wave</param>
		/// <param name="bitmap">bitmap for the sprite</param>
		/// <param name="explodeBitmap">bitmap for the sprite explosion</param>
		/// <param name="hitSound">sound made when a sprite is hit</param>
		/// <param name="explodeSound">sound made when a sprite explodes</param>
		/// <param name="boundingRectangle">bounding rectangle for this wave</param>
		/// <returns>the attackwave produced</returns>
		public static AttackWave DiveBombLeftLow ( 
			int noOfSprites, 
			int spriteValue,
			int spriteGap,
			Bitmap bitmap, 
			Bitmap explodeBitmap,
			Sound hitSound,
			Sound explodeSound,
			Rectangle boundingRectangle ) 
		{
			AttackWave result = new AttackWave (
				noOfSprites,
				spriteValue,
				bitmap,
				explodeBitmap,
				hitSound,
				explodeSound,
				boundingRectangle );

			setStatics ( boundingRectangle, result.Sprites[0] as ScriptedSprite );

			int pos = 0;
			float gap = ( 100 / (float) result.NoOfSprites ) * Xf;

			result.ScriptItems.Add ( new SpriteScriptStepSetVisible ( true ) );
			result.ScriptItems.Add ( new SpriteScriptStepMove( 0, 4 * Yf, 50) );
			result.ScriptItems.Add ( new SppriteScriptStepStop() );

			foreach ( ScriptedSprite sprite in result.Sprites ) 
			{
				sprite.ResetSpriteScriptStep = result.ScriptItems[0] as SpriteScriptStep ;
				sprite.ResetSpritePositionX = (int) ( (pos * gap) + (spriteWidth/2)) ;
				sprite.ResetSpritePositionY = -50 - ( (result.NoOfSprites - pos) * spriteHeight ) ;
				pos++;
			}

			result.ResetWave();

			return result;
		}

		/// <summary>
		/// An attack wave of sprites which move down the screen at speed, the sprites
		/// are in a diagonal with the right side low. 
		/// </summary>
		/// <param name="noOfSprites">number of sprites in the wave</param>
		/// <param name="spriteValue">number of points for each sprite</param>
		/// <param name="spriteGap">gap in steps between each sprite in the wave</param>
		/// <param name="bitmap">bitmap for the sprite</param>
		/// <param name="explodeBitmap">bitmap for the sprite explosion</param>
		/// <param name="hitSound">sound made when a sprite is hit</param>
		/// <param name="explodeSound">sound made when a sprite explodes</param>
		/// <param name="boundingRectangle">bounding rectangle for this wave</param>
		/// <returns>the attackwave produced</returns>
		public static AttackWave DiveBombRightLow ( 
			int noOfSprites, 
			int spriteValue,
			int spriteGap,
			Bitmap bitmap, 
			Bitmap explodeBitmap,
			Sound hitSound,
			Sound explodeSound,
			Rectangle boundingRectangle ) 
		{
			AttackWave result = new AttackWave (
				noOfSprites,
				spriteValue,
				bitmap,
				explodeBitmap,
				hitSound,
				explodeSound,
				boundingRectangle );

			setStatics ( boundingRectangle, result.Sprites[0] as ScriptedSprite );

			int pos = 0;
			float gap = ( 100 / (float) result.NoOfSprites ) * Xf;

			result.ScriptItems.Add ( new SpriteScriptStepSetVisible ( true ) );
			result.ScriptItems.Add ( new SpriteScriptStepMove( 0, 4 * Yf, 50) );
			result.ScriptItems.Add ( new SppriteScriptStepStop() );

			foreach ( ScriptedSprite sprite in result.Sprites ) 
			{
				sprite.ResetSpriteScriptStep = result.ScriptItems[0] as SpriteScriptStep ;
				sprite.ResetSpritePositionX = (int) ( (pos * gap) + (spriteWidth/2)) ;
				sprite.ResetSpritePositionY = -50 - ( pos * spriteHeight ) ;
				pos++;
			}

			result.ResetWave();

			return result;
		}

		/// <summary>
		/// An attack wave of sprites which enter from the top and then search for the
		/// ship, increasing in speed. 
		/// </summary>
		/// <param name="noOfSprites">number of sprites in the wave</param>
		/// <param name="spriteValue">number of points for each sprite</param>
		/// <param name="spriteGap">gap in steps between each sprite in the wave</param>
		/// <param name="bitmap">bitmap for the sprite</param>
		/// <param name="explodeBitmap">bitmap for the sprite explosion</param>
		/// <param name="hitSound">sound made when a sprite is hit</param>
		/// <param name="explodeSound">sound made when a sprite explodes</param>
		/// <param name="boundingRectangle">bounding rectangle for this wave</param>
		/// <returns>the attackwave produced</returns>
		public static AttackWave NastyHomers ( 
			int noOfSprites, 
			int spriteValue,
			int spriteGap,
			Bitmap bitmap, 
			Bitmap explodeBitmap,
			Sound hitSound,
			Sound explodeSound,
			Rectangle boundingRectangle ) 
		{
			AttackWave result = new AttackWave (
				noOfSprites,
				spriteValue,
				bitmap,
				explodeBitmap,
				hitSound,
				explodeSound,
				boundingRectangle );

			setStatics ( boundingRectangle, result.Sprites[0] as ScriptedSprite );

			AttackWave.BuildIntroList ( result, result.NoOfSprites, spriteGap );

			int pos=0;

			foreach ( ScriptedSprite sprite in result.Sprites ) 
			{
				sprite.ResetSpriteScriptStep = result.ScriptItems[pos] as SpriteScriptStep;
				sprite.ResetSpritePositionX = 50 ;
				sprite.ResetSpritePositionY = -50 ;
				pos++;
			}

			result.ScriptItems.Add ( new SpriteScriptStepSetVisible ( true ) );
			result.ScriptItems.Add ( new SpriteScriptStepMove( 0, 2, 25) );
			result.ScriptItems.Add ( new SpriteScriptStepChase ( 1, 10 ) );
			result.ScriptItems.Add ( new SpriteScriptStepChase ( 2, 20 ) );
			result.ScriptItems.Add ( new SpriteScriptStepChase ( 3, 20 ) );
			result.ScriptItems.Add ( new SpriteScriptStepChase ( 4, 30 ) );
			result.ScriptItems.Add ( new SpriteScriptStepMove( -2*Xf, -2*Yf, 50) );
			result.ScriptItems.Add ( new SppriteScriptStepStop() );

			result.ResetWave();

			return result;
		}
        /// <summary>
        /// An attack wave of sprites which enter from the top and then search for the
        /// ship, increasing in speed. 
        /// </summary>
        /// <param name="spriteValue">number of points for each sprite</param>
        /// <param name="bitmap">bitmap for the sprite</param>
        /// <param name="explodeBitmap">bitmap for the sprite explosion</param>
        /// <param name="hitSound">sound made when a sprite is hit</param>
        /// <param name="explodeSound">sound made when a sprite explodes</param>
        /// <param name="boundingRectangle">bounding rectangle for this wave</param>
        /// <returns>the attackwave produced</returns>
        public static AttackWave NastyBoss(
            int spriteValue,
            Bitmap bitmap,
            Bitmap explodeBitmap,
            Sound hitSound,
            Sound explodeSound,
            Rectangle boundingRectangle)
        {
            AttackWave result = new AttackWave(
                spriteValue,
                bitmap,
                explodeBitmap,
                hitSound,
                explodeSound,
                boundingRectangle);

            setStatics(boundingRectangle, result.Sprites[0] as ScriptedSprite);

            ScriptedSprite boss = result.Sprites[0] as ScriptedSprite;

            result.ScriptItems.Add(new SpriteScriptStepPlaceSprite( 0, -boss.SpriteRectangle.Height));
            result.ScriptItems.Add(new SpriteScriptStepSetVisible(true));
            result.ScriptItems.Add(new SpriteScriptStepChase(1, 10));
            result.ScriptItems.Add(new SpriteScriptStepChase(2, 20));
            result.ScriptItems.Add(new SpriteScriptStepChase(3, 200));
            result.ScriptItems.Add(new SpriteScriptStepChase(4, 10));
            result.ScriptItems.Add(new SpriteScriptStepMove(-2 * Xf, -2 * Yf, 50));
            result.ScriptItems.Add(new SppriteScriptStepStop());

            boss.ResetSpriteScriptStep = result.ScriptItems[0] as SpriteScriptStep;

            result.ResetWave();

            return result;
        }
    }



    #endregion

	/// <summary>
	/// An entire game level. Holds a number of waves which are played at the same
	/// time.
	/// </summary>
	public class GameLevel
	{
		/// <summary>
		/// All the attack waves in this level.
		/// </summary>
		public ArrayList Waves = new ArrayList();

		/// <summary>
		/// Resets the level by resetting each fo the waves in it.
		/// </summary>
		public void ResetLevel () 
		{
			foreach ( AttackWave wave in Waves ) 
			{
				wave.ResetWave();
			}
		}

		/// <summary>
		/// Checks to see if every wave in the level is finished.
		/// </summary>
		/// <returns>true if the level is complete</returns>
		public bool LevelComplete () 
		{
			foreach ( AttackWave wave in Waves ) 
			{
				if ( !wave.WaveDestroyed() ) 
				{
					return false;
				}
			}
			return true;
		}

		/// <summary>
		/// Asks each attack wave in the level if the misile has hit anything in
		/// it.
		/// </summary>
		/// <param name="missile">missile attacking the wave</param>
		/// <returns>the score of the sprite hit, or 0 if no sprite was hit.</returns>
		public int CheckForHitScore ( MissileSprite missile ) 
		{
			int result = 0 ;
			foreach ( AttackWave wave in Waves ) 
			{
				result = wave.CheckForHitScore ( missile ) ;

				if ( result != 0 ) 
				{
					return result;
				}
			}
			return result;

		}

		/// <summary>
		/// Checks to see if the ship has collided with anything in this level
		/// </summary>
		/// <param name="ship">the ship we are testing</param>
		/// <returns>true if a collision occured</returns>
		public bool CheckForShipCollide ( ShipSprite ship ) 
		{
			foreach ( AttackWave wave in Waves ) 
			{
				if ( wave.CheckForShipCollide ( ship ) ) 
				{
					return true;
				}
			}
			return false;
		}

		/// <summary>
		/// Constructs level 1
		/// </summary>
		/// <param name="spriteBitmaps">An arraylist of bitmaps for sprites</param>
		/// <param name="explodeBitmap">The explode bitmap to use</param>
		/// <param name="hitSound">the hitsound for the sprites in this level</param>
		/// <param name="explodeSound">the explode sound for this level</param>
		/// <param name="boundingRectangle">the bounding rectangle for sprites in the level</param>
		/// <returns>the level created</returns>

		public static GameLevel Level01 ( 
			ArrayList spriteBitmaps, 
			Bitmap explodeBitmap,
			Sound hitSound,
			Sound explodeSound,
			Rectangle boundingRectangle  ) 
		{
			GameLevel result = new GameLevel();

			result.Waves.Add ( AttackWave.ClockSquareDance (
				2,	// no of sprites
				10, // value of each sprite
				20, // spacing of each sprite
				spriteBitmaps[0] as Bitmap,
				explodeBitmap,
				hitSound,
				explodeSound,
				boundingRectangle ) ) ;
			
			return result;
		}

        public static GameLevel LevelNastyBoss (
            Bitmap spriteBitmap,
            Bitmap explodeBitmap,
            Sound hitSound,
            Sound explodeSound,
            Rectangle boundingRectangle)
        {
            GameLevel result = new GameLevel();

            result.Waves.Add ( AttackWave.NastyBoss( 
                1000, 
                spriteBitmap, 
                explodeBitmap,
                hitSound,
                explodeSound,
                boundingRectangle ));

            return result;
        }


		/// <summary>
		/// Constructs level 2
		/// </summary>
		/// <param name="spriteBitmaps">An arraylist of bitmaps for sprites</param>
		/// <param name="explodeBitmap">The explode bitmap to use</param>
		/// <param name="hitSound">the hitsound for the sprites in this level</param>
		/// <param name="explodeSound">the explode sound for this level</param>
		/// <param name="boundingRectangle">the bounding rectangle for sprites in the level</param>
		/// <returns>the level created</returns>
		public static GameLevel Level02 ( 
			ArrayList spriteBitmaps, 
			Bitmap explodeBitmap,
			Sound hitSound,
			Sound explodeSound,
			Rectangle boundingRectangle  ) 
		{
			GameLevel result = new GameLevel();

			result.Waves.Add ( AttackWave.AntiClockSquareDance (
				3,	// no of sprites
				10, // value of each sprite
				20, // spacing of each sprite
				spriteBitmaps[0] as Bitmap,
				explodeBitmap,
				hitSound,
				explodeSound,
				boundingRectangle ) ) ;
			
			return result;
		}

		/// <summary>
		/// Constructs level 3
		/// </summary>
		/// <param name="spriteBitmaps">An arraylist of bitmaps for sprites</param>
		/// <param name="explodeBitmap">The explode bitmap to use</param>
		/// <param name="hitSound">the hitsound for the sprites in this level</param>
		/// <param name="explodeSound">the explode sound for this level</param>
		/// <param name="boundingRectangle">the bounding rectangle for sprites in the level</param>
		/// <returns>the level created</returns>
		public static GameLevel Level03 ( 
			ArrayList spriteBitmaps, 
			Bitmap explodeBitmap,
			Sound hitSound,
			Sound explodeSound,
			Rectangle boundingRectangle  ) 
		{
			GameLevel result = new GameLevel();

			result.Waves.Add ( AttackWave.AntiClockSquareDanceMobAttack (
				4,	// no of sprites
				10, // value of each sprite
				20, // spacing of each sprite
				spriteBitmaps[0] as Bitmap,
				explodeBitmap,
				hitSound,
				explodeSound,
				boundingRectangle ) ) ;
			
			result.Waves.Add ( AttackWave.NastyHomers (
				3,	// no of sprites
				10, // value of each sprite
				40, // spacing of each sprite
				spriteBitmaps[2] as Bitmap,
				explodeBitmap,
				hitSound,
				explodeSound,
				boundingRectangle ) ) ;
			
			return result;
		}

		/// <summary>
		/// Constructs level 4
		/// </summary>
		/// <param name="spriteBitmaps">An arraylist of bitmaps for sprites</param>
		/// <param name="explodeBitmap">The explode bitmap to use</param>
		/// <param name="hitSound">the hitsound for the sprites in this level</param>
		/// <param name="explodeSound">the explode sound for this level</param>
		/// <param name="boundingRectangle">the bounding rectangle for sprites in the level</param>
		/// <returns>the level created</returns>
		public static GameLevel Level04 ( 
			ArrayList spriteBitmaps, 
			Bitmap explodeBitmap,
			Sound hitSound,
			Sound explodeSound,
			Rectangle boundingRectangle  ) 
		{
			GameLevel result = new GameLevel();

			result.Waves.Add ( AttackWave.DiveBombLeftLow (
				3,	// no of sprites
				10, // value of each sprite
				20, // spacing of each sprite
				spriteBitmaps[1] as Bitmap,
				explodeBitmap,
				hitSound,
				explodeSound,
				boundingRectangle ) ) ;
			return result;
		}

		/// <summary>
		/// Constructs level 5
		/// </summary>
		/// <param name="spriteBitmaps">An arraylist of bitmaps for sprites</param>
		/// <param name="explodeBitmap">The explode bitmap to use</param>
		/// <param name="hitSound">the hitsound for the sprites in this level</param>
		/// <param name="explodeSound">the explode sound for this level</param>
		/// <param name="boundingRectangle">the bounding rectangle for sprites in the level</param>
		/// <returns>the level created</returns>
		public static GameLevel Level05 ( 
			ArrayList spriteBitmaps, 
			Bitmap explodeBitmap,
			Sound hitSound,
			Sound explodeSound,
			Rectangle boundingRectangle  ) 
		{
			GameLevel result = new GameLevel();

			result.Waves.Add ( AttackWave.DiveBombRightLow (
				4,	// no of sprites
				10, // value of each sprite
				20, // spacing of each sprite
				spriteBitmaps[1] as Bitmap,
				explodeBitmap,
				hitSound,
				explodeSound,
				boundingRectangle ) ) ;
			return result;
		}

		/// <summary>
		/// Constructs level 6
		/// </summary>
		/// <param name="spriteBitmaps">An arraylist of bitmaps for sprites</param>
		/// <param name="explodeBitmap">The explode bitmap to use</param>
		/// <param name="hitSound">the hitsound for the sprites in this level</param>
		/// <param name="explodeSound">the explode sound for this level</param>
		/// <param name="boundingRectangle">the bounding rectangle for sprites in the level</param>
		/// <returns>the level created</returns>
		public static GameLevel Level06 ( 
			ArrayList spriteBitmaps, 
			Bitmap explodeBitmap,
			Sound hitSound,
			Sound explodeSound,
			Rectangle boundingRectangle  ) 
		{
			GameLevel result = new GameLevel();

			result.Waves.Add ( AttackWave.DiveBombFlat (
				5,	// no of sprites
				10, // value of each sprite
				20, // spacing of each sprite
				spriteBitmaps[1] as Bitmap,
				explodeBitmap,
				hitSound,
				explodeSound,
				boundingRectangle ) ) ;
			return result;
		}
	}
}
