using System;
using System.Collections.Generic;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using ImageProcess;

namespace PlayWithFriends
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();
            init();

            this.Activated += new EventHandler(this.OnActivate);
            this.Deactivate += new EventHandler(this.OnDeActivate);

        }

        #region Game code

        #region Game variables

        /// <summary>
        /// The background star sprites are held here. They are added to the playfield
        /// for redraw by the draw action.
        /// </summary>
        private ArrayList BackgroundStarSprites = new ArrayList();

        /// <summary>
        /// The player sprites are held here. These include the ship, missiles and 
        /// power ups.
        /// </summary>
        private ArrayList PlayerSprites = new ArrayList();

        /// <summary>
        /// Playfield upon which all the games are drawn 
        /// </summary>
        private PlayField playField;

        /// <summary>
        /// Explosion bitmap used by everything which explodes..
        /// </summary>
        private Bitmap explosionBitmap;

        /// <summary>
        /// All the alien sprite bitmaps
        /// </summary>
        private ArrayList alienBitmaps = new ArrayList();

        /// <summary>
        /// All the levels in the game
        /// </summary>
        private ArrayList gameLevels = new ArrayList();

        /// <summary>
        /// Level currently being played
        /// </summary>
        private GameLevel currentLevel = null;

        /// <summary>
        /// Current score value.
        /// </summary>
        private int score;

        /// <summary>
        /// Sprite which is the player's ship
        /// </summary>
        private ShipSprite ship;

        /// <summary>
        /// Single missile.
        /// </summary>
        public MissileSprite missile;

        /// <summary>
        /// Currently executing assembly. Cached because it is used
        /// to load a lot of resources. 
        /// </summary>
        readonly System.Reflection.Assembly assembly =
            System.Reflection.Assembly.GetExecutingAssembly();

        /// <summary>
        /// Sound made when a missile hits a target but does not destroy it
        /// </summary>
        private Sound hitSound;
        /// <summary>
        /// Sound made when something explodes.
        /// </summary>
        private Sound explodeSound;

        /// <summary>
        /// Sound made when a missile is fired
        /// </summary>
        private Sound fireSound;

        /// <summary>
        /// The keycode of the hey being held down.
        /// </summary>
        private System.Windows.Forms.Keys keyCode;
        /// <summary>
        /// The speed of the ship.
        /// </summary>
        private int shipMoveSpeed = 1;

        /// <summary>
        /// Maximum ship move speed. May increase if we get a powerup
        /// which increases the ship speed. Does not change in this
        /// version of the game.
        /// </summary>
        private int maxShipMoveSpeed = 6;

        /// <summary>
        /// Set to true when a key is being held down. keyCode
        /// holds the code of the key being held down.
        /// </summary>
        private bool keyHeldDown = false;

        /// <summary>
        /// States the game can occupy
        /// </summary>
        enum GameStates
        {
            attract,
            playing
        }

        /// <summary>
        /// Game state control variable
        /// </summary>
        GameStates state = GameStates.attract;

        #endregion

        #region Utility methods

        /// <summary>
        /// Returns a bitmap loaded from the resource.
        /// </summary>
        /// <param name="name">The name of the resource. The path to the
        /// resource is not required</param>
        /// <returns>the bitmap as found</returns>
        private Bitmap getBitmapResource(string name)
        {
            string fullName = Settings.ImageResourceLoc + name;
            Stream bitmapStream = assembly.GetManifestResourceStream(fullName);
            if (bitmapStream == null)
            {
                System.Windows.Forms.MessageBox.Show("Missing file : " + fullName, "Bitmap Load");
                return null;
            }
            return new Bitmap(bitmapStream);
        }

        #endregion

        #region Initialisation methods

        /// <summary>
        /// Makes a new playfield. Decides whether to make a plain on or not
        /// on the basis of the content of the background image filename setting
        /// </summary>
        private void initPlayField()
        {
            Bitmap rawBackground = getBitmapResource(Settings.BackgroundImageFilename);

            float rawAspect = (float)rawBackground.Height / (float)rawBackground.Width;
            int backgroundHeight = (int)((float)ClientRectangle.Width * rawAspect);

            Bitmap background = new Bitmap(ClientRectangle.Width, backgroundHeight);

            using (Graphics g = Graphics.FromImage(background))
            {
                g.DrawImage(
                    rawBackground,
                    new Rectangle(0, 0, background.Width, background.Height),
                    new Rectangle(0, 0, rawBackground.Width, rawBackground.Height),
                    GraphicsUnit.Pixel);
            }

            playField = new PlayField(background, this.ClientRectangle);
            playField.Clear();
            this.Invalidate();
        }

        /// <summary>
        /// Create the background stars and add them to the playfield
        /// </summary>
        private void initBackgroundStars()
        {
            // clear the container
            BackgroundStarSprites.Clear();

            // Add the stars to the container
            for (int i = 0; i < Settings.NoOfBackgroundStars; i++)
            {
                BackgroundStarSprites.Add(
                    new BackgroundStarSprite(this.ClientRectangle));
            }

            // Add the colored stars to the container
            for (int i = 0; i < Settings.NoOfColoredBackgroundStars; i++)
            {
                BackgroundStarSprites.Add(
                    new ColoredBackgroundStarSprite(this.ClientRectangle));
            }
            // Add the star container to the playfield sprites
            playField.GameSpriteLists.Add(BackgroundStarSprites);
        }

        /// <summary>
        /// Sets up the ship and missile objects. 
        /// </summary>
        private void initShipAndMissiles()
        {
            // clear the container
            PlayerSprites.Clear();

            ship = new ShipSprite(
                getBitmapResource(Settings.ShipImageFilename),
                explosionBitmap,
                hitSound,
                explodeSound,
                this.ClientRectangle);

            missile = new MissileSprite(
                new SolidBrush(Color.Yellow),
                fireSound,
                new Rectangle(0, 0, 3, 6),
                this.ClientRectangle);

            PlayerSprites.Add(ship);
            PlayerSprites.Add(missile);

            // tell the script engine about the ship and missile items
            ScriptedSprite.missile = missile;
            ScriptedSprite.ship = ship;

            playField.GameSpriteLists.Add(PlayerSprites);

        }

        /// <summary>
        /// Creates instances of all the game levels and loads them into the levels
        /// store.
        /// </summary>
        private void initGameLevels()
        {
            gameLevels.Add(GameLevel.Level01(
                alienBitmaps,
                explosionBitmap,
                hitSound,
                explodeSound,
                this.ClientRectangle));

            gameLevels.Add(GameLevel.LevelNastyBoss(
                alienBitmaps[3] as Bitmap,
                explosionBitmap,
                hitSound,
                explodeSound,
                this.ClientRectangle));

            gameLevels.Add(GameLevel.Level02(
                alienBitmaps,
                explosionBitmap,
                hitSound,
                explodeSound,
                this.ClientRectangle));

            gameLevels.Add(GameLevel.Level03(
                alienBitmaps,
                explosionBitmap,
                hitSound,
                explodeSound,
                this.ClientRectangle));

            gameLevels.Add(GameLevel.Level04(
                alienBitmaps,
                explosionBitmap,
                hitSound,
                explodeSound,
                this.ClientRectangle));

            gameLevels.Add(GameLevel.Level05(
                alienBitmaps,
                explosionBitmap,
                hitSound,
                explodeSound,
                this.ClientRectangle));

            gameLevels.Add(GameLevel.Level06(
                alienBitmaps,
                explosionBitmap,
                hitSound,
                explodeSound,
                this.ClientRectangle));
        }

        /// <summary>
        /// Loads all the images and sounds and then calls the init methods
        /// for each of the game components.
        /// </summary>
        private void init()
        {
            explosionBitmap = getBitmapResource(Settings.ExplosionImageFilename);
            Bitmap alien01 = getBitmapResource("alien01.png");
            alienBitmaps.Add(alien01);
            Bitmap alien02 = getBitmapResource("alien02.png");
            alienBitmaps.Add(alien02);
            Bitmap alien03 = getBitmapResource("alien03.png");
            alienBitmaps.Add(alien03);
            Bitmap alien04 = getBitmapResource("boss.bmp");
            alienBitmaps.Add(alien04);

            hitSound = new Sound(Settings.SoundResourceLoc + Settings.DamageSound);
            explodeSound = new Sound(Settings.SoundResourceLoc + Settings.DestroyedSound);
            fireSound = new Sound(Settings.SoundResourceLoc + Settings.ShotSound);
            initPlayField();
            initBackgroundStars();
            initShipAndMissiles();
            initGameLevels();

            timer.Enabled = true;
        }

        private void getPlayerImage()
        {
            timer.Enabled = false;

            string imageFile = Utilities.loadPicture(this);

            if (imageFile == null) 
            {
                return;
            }

            Bitmap sprite = Utilities.loadImage(imageFile);

            if (sprite == null)
            {
                return;
            }

            MakeSpriteForm makeSprite =
                new MakeSpriteForm(sprite, this.Width / 3);

            makeSprite.ShowDialog();

            this.Show();

            if ( makeSprite.Result == null ) {
                timer.Enabled = true;
                return;
            }

            // remove the previous boss picture
            alienBitmaps.RemoveAt(3);

            // add our new one
            alienBitmaps.Add(makeSprite.Result);

            GameLevel bossLevel = gameLevels[1] as GameLevel;

            AttackWave bossWave = bossLevel.Waves[0] as AttackWave ;

            ImageAlienSprite alienSprite = bossWave.Sprites[0] as ImageAlienSprite;

            alienSprite.SpriteBitmap = makeSprite.Result;

            timer.Enabled = true;
        }

        #endregion

        #region Game management methods

        /// <summary>
        /// Removes a level from the playfield sprites.
        /// </summary>
        /// <param name="level">The level to be removed.</param>
        private void removeLevel(GameLevel level)
        {
            // clear the background
            playField.Clear();

            if (level == null)
            {
                return;
            }
            // Remove the waves from the playfield
            foreach (AttackWave wave in level.Waves)
            {
                playField.GameSpriteLists.Remove(wave.Sprites);
            }
        }

        /// <summary>
        /// Putes a level onto the screen. Copies the sprites into the playfield.
        /// </summary>
        /// <param name="level">the level to be added to the screen</param>
        private void activateLevel(GameLevel level)
        {
            if (level == null)
            {
                return;
            }

            level.ResetLevel();

            // add the waves to the playfield
            foreach (AttackWave wave in level.Waves)
            {
                playField.GameSpriteLists.Add(wave.Sprites);
            }
        }

        /// <summary>
        /// Starts a brand new game. Resets the levels, lives and score counter.
        /// </summary>
        private void newGame()
        {
            score = 0;
            updateScoreDisplay();
            removeLevel(currentLevel);
            currentLevel = gameLevels[0] as GameLevel;
            currentLevel.ResetLevel();
            ship.Reset();
            playField.XOffset = 0;
            playField.YOffset = 0;
            activateLevel(currentLevel);
            state = GameStates.playing;
        }

        /// <summary>
        /// Moves on to the next level
        /// </summary>
        private void levelUp()
        {
            removeLevel(currentLevel);
            int pos = gameLevels.IndexOf(currentLevel);
            pos++;
            if (pos == gameLevels.Count)
            {
                pos = 0;
            }
            currentLevel = gameLevels[pos] as GameLevel;
            activateLevel(currentLevel);
        }

        /// <summary>
        /// Moves on to the next level if the current one is complete.
        /// </summary>
        private void checkLevelStatus()
        {
            if (currentLevel.LevelComplete())
            {
                levelUp();
            }
        }

        /// <summary>
        /// Updates the score on the top line of the form.
        /// </summary>
        private void updateScoreDisplay()
        {
            this.Text = "StarLight Score : " + score;
        }

        /// <summary>
        /// Checks to see if the missile has hit anything.
        /// </summary>
        private void checkMissile()
        {
            if (missile.State == MissileSpriteStates.flying)
            {
                int hitScore;
                hitScore = currentLevel.CheckForHitScore(missile);
                if (hitScore > 0)
                {
                    score += hitScore;
                    updateScoreDisplay();
                }
            }
        }

        /// <summary>
        /// Looks for ship collisions with the level. Will end the
        /// game if we have been destroyed.
        /// </summary>
        private void checkShip()
        {
            currentLevel.CheckForShipCollide(ship);

            if (ship.State == ShipSpriteStates.invisible)
            {
                state = GameStates.attract;
            }
        }

        #endregion

        #region Game Event methods

        /// <summary>
        /// Moves the ship in a particular direction
        /// </summary>
        /// <param name="keyCode">the keycode being pressed</param>
        /// <param name="speed">the number of pixels to move</param>
        public void MoveShip(System.Windows.Forms.Keys keyCode, int speed)
        {
            switch (keyCode)
            {
                case System.Windows.Forms.Keys.D8:
                case System.Windows.Forms.Keys.Down:
                    ship.YUpdateValue += speed;
                    break;
                case System.Windows.Forms.Keys.D2:
                case System.Windows.Forms.Keys.Up:
                    ship.YUpdateValue -= speed;
                    break;
                case System.Windows.Forms.Keys.D4:
                case System.Windows.Forms.Keys.Left:
                    ship.XUpdateValue -= speed;
                    break;
                case System.Windows.Forms.Keys.D6:
                case System.Windows.Forms.Keys.Right:
                    ship.XUpdateValue += speed;
                    break;
                case System.Windows.Forms.Keys.D1:
                    ship.XUpdateValue -= speed;
                    ship.YUpdateValue -= speed;
                    break;
                case System.Windows.Forms.Keys.D3:
                    ship.XUpdateValue += speed;
                    ship.YUpdateValue -= speed;
                    break;
                case System.Windows.Forms.Keys.D7:
                    ship.XUpdateValue -= speed;
                    ship.YUpdateValue += speed;
                    break;
                case System.Windows.Forms.Keys.D9:
                    ship.XUpdateValue += speed;
                    ship.YUpdateValue += speed;
                    break;
                case System.Windows.Forms.Keys.D5:
                    fireMissile();
                    break;
            }
        }

        /// <summary>
        /// Called when a key goes down (i.e. not when a keypress is complete)
        /// </summary>
        /// <param name="inKeyCode">the code of the key going down</param>
        private void keyDown(System.Windows.Forms.Keys inKeyCode)
        {
            keyHeldDown = true;
            keyCode = inKeyCode;
        }

        /// <summary>
        /// Called when a key is released. Clears the flag and puts the speed 
        /// down to 1 again.
        /// </summary>
        private void keyUp()
        {
            keyHeldDown = false;
            shipMoveSpeed = 1;
        }

        /// <summary>
        /// Moves the ship. Controlled by the keycode. Also increases the speed,
        /// up to the maximum.
        /// </summary>
        private void updateShipPosition()
        {
            if (keyHeldDown)
            {
                MoveShip(keyCode, shipMoveSpeed);
                if (shipMoveSpeed < maxShipMoveSpeed)
                {
                    shipMoveSpeed++;
                }
            }
        }

        /// <summary>
        /// Fires the missile, but only if the game is active.
        /// </summary>
        private void fireMissile()
        {
            if (state == GameStates.playing)
            {
                missile.Fire(
                    ship.X +
                    (ship.SpriteRectangle.Width / 2) -
                    (missile.SpriteRectangle.Width / 2),
                    ship.Y -
                    missile.SpriteRectangle.Height
                    );
            }
        }

        /// <summary>
        /// Called to deal with a key press. Note that this is 
        /// a completed press, i.e. key down and up again.
        /// </summary>
        /// <param name="key">the ASCII code of the key</param>
        private void keyPress(char key)
        {
            if (key == 0x0d)
            {
                fireMissile();
                return;
            }
        }

        /// <summary>
        /// Called by the timer to update the game each timer tick
        /// </summary>
        private void updateGame()
        {
            switch (state)
            {

                case GameStates.playing:
                    updateShipPosition();
                    checkMissile();
                    checkShip();
                    checkLevelStatus();
                    break;

                case GameStates.attract:
                    break;
            }

            playField.UpdateGame();
            this.Invalidate();
        }

        private void paint(Graphics g)
        {
            playField.Paint(g);
        }

        /// <summary>
        /// Called to pause the game if focus is lost from this form
        /// </summary>
        private void pauseGame()
        {
            Sound.StopSound();
            timer.Enabled = false;
        }

        /// <summary>
        /// Called to resume the game if focus returns to it.
        /// </summary>
        private void resumeGame()
        {
            Sound.ResumeSound();
            timer.Enabled = true;
        }

        protected override void OnPaintBackground(PaintEventArgs e)
        {
            // don't let the background get painted
        }


        #endregion


        #endregion

        private void timer1_Tick(object sender, EventArgs e)
        {
            updateGame();
        }

        private void MainForm_GotFocus(object sender, EventArgs e)
        {
            resumeGame();

        }

        private void MainForm_LostFocus(object sender, EventArgs e)
        {
            pauseGame();
        }

        private void MainForm_KeyDown(object sender, KeyEventArgs e)
        {
            keyDown(e.KeyCode);
        }

        private void MainForm_KeyPress(object sender, KeyPressEventArgs e)
        {
            keyPress(e.KeyChar);
        }

        private void MainForm_KeyUp(object sender, KeyEventArgs e)
        {
            keyUp();
        }

        private void MainForm_Paint(object sender, PaintEventArgs e)
        {
            paint(e.Graphics);
        }

        private void leftLKeyMenuItem_Click(object sender, EventArgs e)
        {
            newGame();		
        }

        void OnActivate(object sender, EventArgs e)
        {
            Sound.ResumeSound();
        }

        void OnDeActivate(object sender, EventArgs e)
        {
            Sound.StopSound();
        }

        private void exitMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void playerMenuItem_Click(object sender, EventArgs e)
        {
            getPlayerImage();
        }
    }
}